function [Ds, L, Dc] = rsscale(Do, intervals)
% RSSCALE - Rescale a design matrix. Version 0.1.11.
%
% The syntax is
% [Ds, L, Dc] = rsscale(Do, ints)
%
% Arguments
% Do is the original design matrix. Every factor must take at least two different values.
% ints is an a by 2 matrix, where every row [low_i high_i] describes the natural values low_i and high_i of factor number i in Ds. These are the values into which the smallest and largest, respectively, values of factor i in Do is mapped into.
%
% Return values
% Ds is the design matrix Do scaled to take the minimum and maximum values specified in the matrix "intervals".
% L is the scaling matrix such that Xs=Xo*L, where Xs is Ds with an N by 1 column vector of ones added at the left and Xo is obtained from Do in the same way.
% Dc is coded in such a way that the maximum value in every column of Do is mapped to +1 and the minimum value into -1.

% References
% No reference needed, this is trivial.


% Copyright (C) 2015 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin ~= 2 ,
    error('Try help rsscale for instructions.') ;
end
if size(intervals,2) ~= 2,
    error('The matrix intervals has the wrong number of columns. Try help rsscale for instructions.');
end
if size(intervals,1) ~= size(Do,2),
    error('The number of factors in Do and ints disagree. Try help rsscale for instructions.');
end

% First scale to +/- 1 coded data matrix Xc
[N, k]=size(Do);
Xo=[ones(N,1) Do];
minmax=[min(Xo(:,2:end))' max(Xo(:,2:end))'];

onelevel=size(find(abs(diff(minmax'))<1e-10));

if onelevel(2)>0,
    error('At least one factor seems to take only one value. This factor should be removed from the design matrix. If this is not the case, try standardizing your data.')
end

ell=minmax*[1 1]'/2;
p=minmax*[-1 1]'/2;
Lc=[ 1 ell' ; zeros(k,1) diag(p)]^-1;
Xc=Xo*Lc;
Dc=Xc(:,2:end);

% Now scale Xc as requested for Ds
ell=intervals*[1 1]'/2;
p=intervals*[-1 1]'/2;
Ls=[ 1 ell' ; zeros(k,1) diag(p)];
Xs=Xc*Ls;
Ds=Xs(:,2:end);
L=Lc*Ls;

end
